@@ -41,6 +41,8 @@ const char* Harmonizer::VCF_VARIANT_TYPE = "VT";
 const char* Harmonizer::VCF_SNP_TYPE = "SNP";
 const char* Harmonizer::VCF_INDEL_TYPE_01 = "INDEL";
 const char* Harmonizer::VCF_INDEL_TYPE_02 = "I";
+const char* Harmonizer::VCF_ALT_ALLELE_DEL = "<DEL>";
+const char* Harmonizer::VCF_ALT_ALLELE_INS = "<INS>";
 
 const unsigned int Harmonizer::MAP_HEAP_SIZE = 3000000;
 const unsigned int Harmonizer::MAP_HEAP_INCREMENT = 1000000;
@@ -245,6 +247,31 @@ void Harmonizer::process_map_file_header() throw (HarmonizerException) {
 					break;
 				} else {
 					/* process meta-info line if necessary */
+
+//					if (auxiliary::strcmp_ignore_case(line, "##ALT", 5) == 0) {
+//						token = strstr(line, "ID");
+//						if (token == NULL) {
+//							continue;
+//						}
+//
+//						if (strpbrk(token, ",>") != NULL) {
+//							strpbrk(token, ",>")[0u] = '\0';
+//						}
+//
+//						token = strchr(token, '=');
+//						if (token == NULL) {
+//							continue;
+//						}
+//
+//						++token;
+//
+//						auxiliary::trim(&token);
+//						if (strlen(token) <= 0) {
+//							continue;
+//						}
+//
+//						/* the token contains variant type name */
+//					}
 				}
 			} else {
 				throw HarmonizerException("Harmonizer", "process_map_file_header()", __LINE__, 12, map_file_line_number);
@@ -327,7 +354,9 @@ void Harmonizer::process_map_file_data() throw (HarmonizerException) {
 
 			nonref_allele = tokens[4u];
 			nonref_allele_length = strlen(nonref_allele);
-			if ((nonref_allele_length <= 0u) || (strspn(nonref_allele, "ACGT") != nonref_allele_length)) {
+			if (((nonref_allele_length <= 0u) || (strspn(nonref_allele, "ACGT") != nonref_allele_length)) &&
+					(auxiliary::strcmp_ignore_case(nonref_allele, VCF_ALT_ALLELE_DEL) != 0) &&
+					(auxiliary::strcmp_ignore_case(nonref_allele, VCF_ALT_ALLELE_INS) != 0)) {
 				continue;
 			}
 
@@ -398,8 +427,9 @@ void Harmonizer::process_map_file_data() throw (HarmonizerException) {
 			} else {
 				vt_found = false;
 			}
-			/* END: determine variant type (only I and S are supported) */
 
+			/* in all other cases it is I (indel) */
+			/* END: determine variant type (only I and S are supported) */
 
 			if (index->n >= index->heap_n) {
 				index->heap_n += MAP_HEAP_INCREMENT;
@@ -773,38 +803,28 @@ void Harmonizer::process_header() throw (HarmonizerException) {
 		}
 		strcpy(header_backup, header);
 
-		if ((ref_allele_column != NULL) && (nonref_allele_column != NULL)) {
-			while ((token = auxiliary::strtok(&header, separator)) != NULL) {
-				if (auxiliary::strcmp_ignore_case(token, chr_column) == 0) {
-					chr_column_pos = file_column_number;
-				} else if (auxiliary::strcmp_ignore_case(token, id_column) == 0) {
-					id_column_pos = file_column_number;
-				} else if (auxiliary::strcmp_ignore_case(token, ref_allele_column) == 0) {
-					ref_allele_column_pos = file_column_number;
-				} else if (auxiliary::strcmp_ignore_case(token, nonref_allele_column) == 0) {
-					nonref_allele_column_pos = file_column_number;
-				}
-				++file_column_number;
-			}
-
-			if (ref_allele_column_pos < 0) {
-				throw HarmonizerException("Harmonizer", "process_header()", __LINE__, 17, ref_allele_column, input_file);
-			}
-
-			if (nonref_allele_column_pos < 0) {
-				throw HarmonizerException("Harmonizer", "process_header()", __LINE__, 17, nonref_allele_column, input_file);
-			}
-		} else {
-			while ((token = auxiliary::strtok(&header, separator)) != NULL) {
-				if (auxiliary::strcmp_ignore_case(token, chr_column) == 0) {
-					chr_column_pos = file_column_number;
-				} else if (auxiliary::strcmp_ignore_case(token, id_column) == 0) {
-					id_column_pos = file_column_number;
-				}
-				++file_column_number;
+		while ((token = auxiliary::strtok(&header, separator)) != NULL) {
+			if (auxiliary::strcmp_ignore_case(token, chr_column) == 0) {
+				chr_column_pos = file_column_number;
+			} else if (auxiliary::strcmp_ignore_case(token, id_column) == 0) {
+				id_column_pos = file_column_number;
+			} else if (auxiliary::strcmp_ignore_case(token, ref_allele_column) == 0) {
+				ref_allele_column_pos = file_column_number;
+			} else if (auxiliary::strcmp_ignore_case(token, nonref_allele_column) == 0) {
+				nonref_allele_column_pos = file_column_number;
 			}
+			++file_column_number;
 		}
 
+		if (ref_allele_column_pos < 0) {
+			throw HarmonizerException("Harmonizer", "process_header()", __LINE__, 17, ref_allele_column, input_file);
+		}
+
+		if (nonref_allele_column_pos < 0) {
+			throw HarmonizerException("Harmonizer", "process_header()", __LINE__, 17, nonref_allele_column, input_file);
+		}
+
+
 		if (chr_column_pos < 0) {
 			throw HarmonizerException("Harmonizer", "process_header()", __LINE__, 17, chr_column, input_file);
 		}
@@ -863,6 +883,8 @@ void Harmonizer::harmonize(bool drop) throw (HarmonizerException) {
 	char* buffer_tokens[3u];
 	char* end_ptr = NULL;
 
+	bool empty_ref_allele = false;
+	bool empty_nonref_allele = false;
 	bool recode_alleles = false;
 	bool consistent_type = false;
 	bool consistent_alleles = false;
@@ -1118,6 +1140,120 @@ void Harmonizer::harmonize(bool drop) throw (HarmonizerException) {
 					}
 				}
 
+				found_position = &(position_index[id_index[position].location]);
+				if (found_position->type == 'S') {
+					sprintf(buffer, "%s:%lu:%s", tokens[chr_column_pos], found_position->position, VCF_SNP_TYPE);
+				} else {
+					sprintf(buffer, "%s:%lu:%s", tokens[chr_column_pos], found_position->position, VCF_INDEL_TYPE_01);
+				}
+				log_writer->write("Line %u: %s changed to %s.\n", line_number, tokens[id_column_pos], buffer);
+				tokens[id_column_pos] = buffer;
+			} else if (auxiliary::strcmp_ignore_case(tokens[id_column_pos], "merged_del", 10) == 0) { // Look up by id
+				query_id.id = tokens[id_column_pos];
+				found_id = (id_index_entry*)bsearch(&query_id, id_index, index_size, sizeof(id_index_entry), qsort_id_index_entry_cmp);
+				if (found_id == NULL) {
+					log_writer->write("Line %u: (WARNING) %s is not in the map.\n", line_number, query_id.id);
+					if (!drop) write_columns();
+					continue;
+				}
+
+				if (((auxiliary::strcmp_ignore_case(ref_allele, "D") == 0) && (auxiliary::strcmp_ignore_case(nonref_allele, "R") == 0)) ||
+							((auxiliary::strcmp_ignore_case(ref_allele, "R") == 0) && (auxiliary::strcmp_ignore_case(nonref_allele, "D") == 0))) {
+					query_id.type = 'I';
+				} else {
+					query_id.type = 'I';
+
+					empty_ref_allele = false;
+					empty_nonref_allele = false;
+
+					if ((strcmp(ref_allele, "-") == 0) || (strcmp(ref_allele, ".") == 0) || (auxiliary::strcmp_ignore_case(ref_allele, "NA") == 0)) {
+						empty_ref_allele = true;
+					} else if (strspn(ref_allele, "ACGT") != ref_allele_length) {
+						log_writer->write("Line %u: (WARNING) Alleles of %s don't match the specified variation type.\n", line_number, tokens[id_column_pos]);
+						if (!drop) write_columns();
+						continue;
+					}
+
+					if ((strcmp(nonref_allele, "-") == 0) || (strcmp(nonref_allele, ".") == 0) || (auxiliary::strcmp_ignore_case(nonref_allele, "NA") == 0)) {
+						empty_nonref_allele = true;
+					} else if (strspn(nonref_allele, "ACGT") != nonref_allele_length) {
+						log_writer->write("Line %u: (WARNING) Alleles of %s don't match the specified variation type.\n", line_number, tokens[id_column_pos]);
+						if (!drop) write_columns();
+						continue;
+					}
+
+					if (empty_ref_allele && empty_nonref_allele) {
+						log_writer->write("Line %u: (WARNING) Alleles of %s don't match the specified variation type.\n", line_number, tokens[id_column_pos]);
+						if (!drop) write_columns();
+						continue;
+					}
+
+					if (!empty_ref_allele && !empty_nonref_allele && (ref_allele_length == nonref_allele_length)) {
+						log_writer->write("Line %u: (WARNING) Alleles of %s don't match the specified variation type.\n", line_number, tokens[id_column_pos]);
+						if (!drop) write_columns();
+						continue;
+					}
+
+					if (empty_ref_allele) {
+						log_writer->write("Line %u: Alleles %s/%s of %s changed to %s/%s.\n", line_number, ref_allele, nonref_allele, tokens[id_column_pos], "D", "R");
+						tokens[ref_allele_column_pos][0] = 'D';
+						tokens[ref_allele_column_pos][1] = '\0';
+						tokens[nonref_allele_column_pos][0] = 'R';
+						tokens[nonref_allele_column_pos][1] = '\0';
+					} else if (empty_nonref_allele) {
+						log_writer->write("Line %u: Alleles %s/%s of %s changed to %s/%s.\n", line_number, ref_allele, nonref_allele, tokens[id_column_pos], "R", "D");
+						tokens[ref_allele_column_pos][0] = 'R';
+						tokens[ref_allele_column_pos][1] = '\0';
+						tokens[nonref_allele_column_pos][0] = 'D';
+						tokens[nonref_allele_column_pos][1] = '\0';
+					} else if (ref_allele_length > nonref_allele_length) {
+						log_writer->write("Line %u: Alleles %s/%s of %s changed to %s/%s.\n", line_number, ref_allele, nonref_allele, tokens[id_column_pos], "R", "D");
+						tokens[ref_allele_column_pos][0] = 'R';
+						tokens[ref_allele_column_pos][1] = '\0';
+						tokens[nonref_allele_column_pos][0] = 'D';
+						tokens[nonref_allele_column_pos][1] = '\0';
+					} else if (ref_allele_length < nonref_allele_length) {
+						log_writer->write("Line %u: Alleles %s/%s of %s changed to %s/%s.\n", line_number, ref_allele, nonref_allele, tokens[id_column_pos], "D", "R");
+						tokens[ref_allele_column_pos][0] = 'D';
+						tokens[ref_allele_column_pos][1] = '\0';
+						tokens[nonref_allele_column_pos][0] = 'R';
+						tokens[nonref_allele_column_pos][1] = '\0';
+					} else {
+						log_writer->write("Line %u: (WARNING) Alleles of %s don't match the specified variation type.\n", line_number, tokens[id_column_pos]);
+						if (!drop) write_columns();
+						continue;
+					}
+				}
+
+				found_id_index_entry_pos = found_id - id_index;
+
+				position = found_id_index_entry_pos;
+				while ((position >= 0) && (qsort_id_index_entry_cmp(&(id_index[position]), &query_id) == 0)) {
+					if (position_index[id_index[position].location].type == query_id.type) {
+						consistent_type = true;
+						break;
+					}
+					--position;
+				}
+
+				if (!consistent_type) {
+					position = found_id_index_entry_pos;
+					++position;
+					while ((position < index_size) && (qsort_id_index_entry_cmp(&(id_index[position]), &query_id) == 0)) {
+						if (position_index[id_index[position].location].type == query_id.type) {
+							consistent_type = true;
+							break;
+						}
+						++position;
+					}
+				}
+
+				if (!consistent_type) {
+					log_writer->write("Line %u: (WARNING) Type of %s doesn't match type in the map.\n", line_number, query_id.id);
+					if (!drop) write_columns();
+					continue;
+				}
+
 				found_position = &(position_index[id_index[position].location]);
 				if (found_position->type == 'S') {
 					sprintf(buffer, "%s:%lu:%s", tokens[chr_column_pos], found_position->position, VCF_SNP_TYPE);
@@ -1475,6 +1611,8 @@ void Harmonizer::harmonize_no_vcf_allele_check(bool drop) throw (HarmonizerExcep
 	char* buffer_tokens[3u];
 	char* end_ptr = NULL;
 
+	bool empty_ref_allele = false;
+	bool empty_nonref_allele = false;
 	bool consistent_type = false;
 
 	if ((input_file == NULL) || (reader == NULL) ||
@@ -1611,6 +1749,120 @@ void Harmonizer::harmonize_no_vcf_allele_check(bool drop) throw (HarmonizerExcep
 					continue;
 				}
 
+				found_position = &(position_index[id_index[position].location]);
+				if (found_position->type == 'S') {
+					sprintf(buffer, "%s:%lu:%s", tokens[chr_column_pos], found_position->position, VCF_SNP_TYPE);
+				} else {
+					sprintf(buffer, "%s:%lu:%s", tokens[chr_column_pos], found_position->position, VCF_INDEL_TYPE_01);
+				}
+				log_writer->write("Line %u: %s changed to %s.\n", line_number, tokens[id_column_pos], buffer);
+				tokens[id_column_pos] = buffer;
+			} else if (auxiliary::strcmp_ignore_case(tokens[id_column_pos], "merged_del", 10) == 0) { // Look up by id
+				query_id.id = tokens[id_column_pos];
+				found_id = (id_index_entry*)bsearch(&query_id, id_index, index_size, sizeof(id_index_entry), qsort_id_index_entry_cmp);
+				if (found_id == NULL) {
+					log_writer->write("Line %u: (WARNING) %s is not in the map.\n", line_number, query_id.id);
+					if (!drop) write_columns();
+					continue;
+				}
+
+				if (((auxiliary::strcmp_ignore_case(ref_allele, "D") == 0) && (auxiliary::strcmp_ignore_case(nonref_allele, "R") == 0)) ||
+							((auxiliary::strcmp_ignore_case(ref_allele, "R") == 0) && (auxiliary::strcmp_ignore_case(nonref_allele, "D") == 0))) {
+					query_id.type = 'I';
+				} else {
+					query_id.type = 'I';
+
+					empty_ref_allele = false;
+					empty_nonref_allele = false;
+
+					if ((strcmp(ref_allele, "-") == 0) || (strcmp(ref_allele, ".") == 0) || (auxiliary::strcmp_ignore_case(ref_allele, "NA") == 0)) {
+						empty_ref_allele = true;
+					} else if (strspn(ref_allele, "ACGT") != ref_allele_length) {
+						log_writer->write("Line %u: (WARNING) Alleles of %s don't match the specified variation type.\n", line_number, tokens[id_column_pos]);
+						if (!drop) write_columns();
+						continue;
+					}
+
+					if ((strcmp(nonref_allele, "-") == 0) || (strcmp(nonref_allele, ".") == 0) || (auxiliary::strcmp_ignore_case(nonref_allele, "NA") == 0)) {
+						empty_nonref_allele = true;
+					} else if (strspn(nonref_allele, "ACGT") != nonref_allele_length) {
+						log_writer->write("Line %u: (WARNING) Alleles of %s don't match the specified variation type.\n", line_number, tokens[id_column_pos]);
+						if (!drop) write_columns();
+						continue;
+					}
+
+					if (empty_ref_allele && empty_nonref_allele) {
+						log_writer->write("Line %u: (WARNING) Alleles of %s don't match the specified variation type.\n", line_number, tokens[id_column_pos]);
+						if (!drop) write_columns();
+						continue;
+					}
+
+					if (!empty_ref_allele && !empty_nonref_allele && (ref_allele_length == nonref_allele_length)) {
+						log_writer->write("Line %u: (WARNING) Alleles of %s don't match the specified variation type.\n", line_number, tokens[id_column_pos]);
+						if (!drop) write_columns();
+						continue;
+					}
+
+					if (empty_ref_allele) {
+						log_writer->write("Line %u: Alleles %s/%s of %s changed to %s/%s.\n", line_number, ref_allele, nonref_allele, tokens[id_column_pos], "D", "R");
+						tokens[ref_allele_column_pos][0] = 'D';
+						tokens[ref_allele_column_pos][1] = '\0';
+						tokens[nonref_allele_column_pos][0] = 'R';
+						tokens[nonref_allele_column_pos][1] = '\0';
+					} else if (empty_nonref_allele) {
+						log_writer->write("Line %u: Alleles %s/%s of %s changed to %s/%s.\n", line_number, ref_allele, nonref_allele, tokens[id_column_pos], "R", "D");
+						tokens[ref_allele_column_pos][0] = 'R';
+						tokens[ref_allele_column_pos][1] = '\0';
+						tokens[nonref_allele_column_pos][0] = 'D';
+						tokens[nonref_allele_column_pos][1] = '\0';
+					} else if (ref_allele_length > nonref_allele_length) {
+						log_writer->write("Line %u: Alleles %s/%s of %s changed to %s/%s.\n", line_number, ref_allele, nonref_allele, tokens[id_column_pos], "R", "D");
+						tokens[ref_allele_column_pos][0] = 'R';
+						tokens[ref_allele_column_pos][1] = '\0';
+						tokens[nonref_allele_column_pos][0] = 'D';
+						tokens[nonref_allele_column_pos][1] = '\0';
+					} else if (ref_allele_length < nonref_allele_length) {
+						log_writer->write("Line %u: Alleles %s/%s of %s changed to %s/%s.\n", line_number, ref_allele, nonref_allele, tokens[id_column_pos], "D", "R");
+						tokens[ref_allele_column_pos][0] = 'D';
+						tokens[ref_allele_column_pos][1] = '\0';
+						tokens[nonref_allele_column_pos][0] = 'R';
+						tokens[nonref_allele_column_pos][1] = '\0';
+					} else {
+						log_writer->write("Line %u: (WARNING) Alleles of %s don't match the specified variation type.\n", line_number, tokens[id_column_pos]);
+						if (!drop) write_columns();
+						continue;
+					}
+				}
+
+				found_id_index_entry_pos = found_id - id_index;
+
+				position = found_id_index_entry_pos;
+				while ((position >= 0) && (qsort_id_index_entry_cmp(&(id_index[position]), &query_id) == 0)) {
+					if (position_index[id_index[position].location].type == query_id.type) {
+						consistent_type = true;
+						break;
+					}
+					--position;
+				}
+
+				if (!consistent_type) {
+					position = found_id_index_entry_pos;
+					++position;
+					while ((position < index_size) && (qsort_id_index_entry_cmp(&(id_index[position]), &query_id) == 0)) {
+						if (position_index[id_index[position].location].type == query_id.type) {
+							consistent_type = true;
+							break;
+						}
+						++position;
+					}
+				}
+
+				if (!consistent_type) {
+					log_writer->write("Line %u: (WARNING) Type of %s doesn't match type in the map.\n", line_number, query_id.id);
+					if (!drop) write_columns();
+					continue;
+				}
+
 				found_position = &(position_index[id_index[position].location]);
 				if (found_position->type == 'S') {
 					sprintf(buffer, "%s:%lu:%s", tokens[chr_column_pos], found_position->position, VCF_SNP_TYPE);
